
# q-expansion of newform 300.5.k.d, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 300
weight = 5
poly_data = [1296, 864, 288, -168, 97, 28, 8, -4, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[-145584, -180684, 175455, -17563, -5404, -4880, 2023, -343], 486540], [[-160812, 579618, -43155, 15391, -15812, 12650, -2731, 331], 162180], [[-579672, -211512, 80710, -55399, -27972, -5960, 3234, -839], 108120], [[1032840, -328788, -85764, 105637, 33580, 1316, -2272, 805], 64872], [[10153296, 13561236, -3558870, 1944877, 367996, 220340, -87082, 19477], 973080], [[485136, -303399, 215496, -31120, 8147, -13697, 4411, -634], 24327], [[4114800, 1334448, -865116, 25003, 114688, 36896, -17932, 3283], 97308]]

hecke_ring_character_values = [[151, [1, 0, 0, 0, 0, 0, 0, 0]], [101, [1, 0, 0, 0, 0, 0, 0, 0]], [277, [0, 1, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [17, 17, 4, 0, 0, 0, 1, 0], [39, 0, 0, 0, 1, 0, -3, -3], [-40, 40, 0, 8, -6, -6, 0, 5], [126, 126, 12, 0, 2, -2, 3, 0], [0, 26, -4, 4, 0, 12, -1, 1], [-161, 161, 0, 20, 11, 11, 0, -8], [0, 471, 96, -96, 0, -19, 6, -6], [170, 0, 80, 80, 42, 0, 14, 14], [60, 60, 144, 0, -6, 6, -45, 0], [-446, 0, 244, 244, -68, 0, 11, 11], [808, -808, 0, 220, -66, -66, 0, -26], [-615, -615, 156, 0, -21, 21, 30, 0], [-505, 505, 0, 508, 25, 25, 0, 35], [0, -967, 260, -260, 0, -7, -41, 41], [-1394, 0, 464, 464, 72, 0, -49, -49], [76, 76, 588, 0, 42, -42, 78, 0], [-388, 0, 548, 548, 152, 0, -62, -62], [-413, 413, 0, 584, 30, 30, 0, 86], [0, -1106, 788, -788, 0, -18, -40, 40], [-1561, 1561, 0, 340, -39, -39, 0, -58], [0, 5128, -44, 44, 0, 58, 17, -17], [1915, 1915, -728, 0, -186, 186, 70, 0], [-1663, 0, 668, 668, -19, 0, 211, 211], [3043, -3043, 0, 500, 408, 408, 0, -121], [299, 299, 628, 0, -271, 271, -178, 0], [0, -508, -360, 360, 0, -786, -138, 138], [-8666, 8666, 0, 644, -236, -236, 0, -23], [5297, 5297, -68, 0, 504, -504, -59, 0], [-11457, 0, -2256, -2256, -59, 0, -141, -141], [-8554, -8554, -740, 0, 204, -204, 563, 0], [0, -11292, 452, -452, 0, 798, 125, -125], [0, 18319, -1532, 1532, 0, 597, -55, 55], [144, 0, -3944, -3944, -306, 0, -89, -89], [708, 708, -2400, 0, -696, 696, 99, 0], [16986, -16986, 0, -1756, -786, -786, 0, 308], [-12893, -12893, -628, 0, 427, -427, -104, 0], [-9252, 9252, 0, -804, 626, 626, 0, -681], [0, 15755, -412, 412, 0, -2129, -173, 173], [700, 0, -248, -248, 1122, 0, 640, 640], [-13826, 0, -3404, -3404, -634, 0, 440, 440], [17549, -17549, 0, -3040, -900, -900, 0, 272], [-17893, -17893, -6428, 0, -939, 939, -829, 0], [0, 15316, 1820, -1820, 0, 786, -7, 7], [1224, 0, -5200, -5200, 858, 0, 251, 251], [-9103, 9103, 0, -5332, -102, -102, 0, -709], [-18390, -18390, 1956, 0, -182, 182, 780, 0], [0, -2796, -2784, 2784, 0, -198, 579, -579], [-28666, 28666, 0, 220, -402, -402, 0, 1727], [0, 40980, 4776, -4776, 0, 2532, -276, 276], [9352, 0, 552, 552, 360, 0, -1224, -1224], [1883, 0, 2240, 2240, 3105, 0, -845, -845], [-8866, -8866, -572, 0, 2824, -2824, 77, 0], [-46987, 46987, 0, 1540, -1391, -1391, 0, 164], [0, 395, -4732, 4732, 0, -2827, 223, -223], [44940, 0, 4968, 4968, -2010, 0, -291, -291], [13562, 13562, -6040, 0, 252, -252, -139, 0], [-59570, 0, 2716, 2716, 684, 0, -679, -679], [16770, -16770, 0, 4004, 696, 696, 0, -490], [12519, -12519, 0, 13596, -375, -375, 0, -1743], [-43110, -43110, 7860, 0, -1548, 1548, 870, 0], [30980, 0, 860, 860, -1408, 0, 1354, 1354], [-283, 283, 0, 11024, 2358, 2358, 0, -874], [-45046, -45046, -2180, 0, 492, -492, -2263, 0], [-96526, 0, 3032, 3032, 396, 0, 383, 383], [-24205, -24205, -4264, 0, 2040, -2040, -940, 0], [-2812, -2812, 11956, 0, 5156, -5156, -1036, 0], [0, 66334, 2400, -2400, 0, 288, -2061, 2061], [9704, -9704, 0, 3220, -768, -768, 0, -463], [0, -44042, 6304, -6304, 0, -2006, 2444, -2444], [15747, 15747, 740, 0, -1602, 1602, 1541, 0], [-37738, 37738, 0, 11040, -2658, -2658, 0, 1761], [0, -11612, 2652, -2652, 0, 4290, 3351, -3351], [20791, -20791, 0, 20444, -2547, -2547, 0, 1198], [0, -124947, -5316, 5316, 0, 51, -2295, 2295], [40044, 40044, 18784, 0, 1452, -1452, 1657, 0], [26546, 0, -8560, -8560, -5340, 0, 190, 190], [0, 49402, -12232, 12232, 0, 5514, -979, 979], [0, -61915, 212, -212, 0, 2865, -3119, 3119], [41152, 0, -7328, -7328, 1686, 0, -767, -767], [116498, 0, -3292, -3292, 3718, 0, -530, -530], [-66651, 66651, 0, -10000, -3390, -3390, 0, -4828], [0, -21612, -6220, 6220, 0, -2178, 3431, -3431], [78658, -78658, 0, -6796, 5670, 5670, 0, 1624], [0, -30544, -11944, 11944, 0, -10750, 1018, -1018], [3597, 3597, -11064, 0, 1932, -1932, -4584, 0], [139787, 0, -892, -892, 3055, 0, -641, -641], [28909, -28909, 0, -11604, 2970, 2970, 0, 27], [80941, 80941, -39340, 0, -3661, 3661, -302, 0], [0, 28220, -14704, 14704, 0, -712, -2498, 2498], [-45903, -45903, -22356, 0, -3996, 3996, -159, 0], [333, 0, -22176, -22176, -8533, 0, 2271, 2271], [0, 77158, -20564, 20564, 0, 372, 49, -49], [69981, -69981, 0, -26292, -1977, -1977, 0, -3882], [0, -236449, 8456, -8456, 0, 5849, 2440, -2440], [-98072, 0, -31160, -31160, 5442, 0, 644, 644], [-7412, 7412, 0, -23252, -3846, -3846, 0, 4714], [-142428, 0, -15128, -15128, -5430, 0, -5294, -5294], [-45634, -45634, 10676, 0, -1032, 1032, 3698, 0], [47395, 47395, 39068, 0, -423, 423, 4735, 0], [-40032, 40032, 0, -6036, -6972, -6972, 0, -3096], [0, -77302, 12032, -12032, 0, 8428, 6580, -6580], [-105374, 0, 496, 496, 8208, 0, -5111, -5111], [-29833, -29833, -38784, 0, 2880, -2880, 126, 0], [164253, 164253, 2316, 0, -4197, 4197, 6534, 0], [249746, -249746, 0, -33332, -2716, -2716, 0, 6263], [0, 144678, -29376, 29376, 0, 10406, 390, -390], [256892, 0, -13464, -13464, -4746, 0, 3471, 3471], [-225159, -225159, -35508, 0, 180, -180, 4773, 0], [-117496, 117496, 0, 19520, 5592, 5592, 0, -2713], [-13430, -13430, -27748, 0, 3162, -3162, -7865, 0], [0, 295044, -3108, 3108, 0, 8394, 2583, -2583], [-165186, 0, 28720, 28720, -23646, 0, 2248, 2248], [-116900, 0, -17756, -17756, 3026, 0, 6029, 6029], [-5992, 5992, 0, -8164, -5184, -5184, 0, -12976], [81837, 81837, 132, 0, 1387, -1387, -11214, 0], [179056, -179056, 0, 50276, 6798, 6798, 0, 1543], [0, -81383, -7340, 7340, 0, -14351, -2773, 2773], [73406, 0, 39792, 39792, 2928, 0, -2733, -2733], [28977, -28977, 0, -37040, 1524, 1524, 0, 2806], [-111354, -111354, 26532, 0, 2432, -2432, -837, 0], [-12641, 12641, 0, 112940, 7305, 7305, 0, 1102], [-22082, 0, 45032, 45032, 9372, 0, 6605, 6605], [63315, 0, 48720, 48720, -17117, 0, 3636, 3636], [0, 145624, 72912, -72912, 0, 2034, 4713, -4713], [0, 148268, -1000, 1000, 0, 12152, 706, -706], [10649, 10649, -23236, 0, 9948, -9948, 7697, 0], [133870, -133870, 0, 78152, -11436, -11436, 0, 3635], [0, -258086, -52172, 52172, 0, 17424, -3119, 3119], [-65181, 65181, 0, 16428, -7091, -7091, 0, 21882], [-496266, 0, 10536, 10536, -3210, 0, -3462, -3462], [143292, 143292, 84456, 0, -1302, 1302, -4599, 0], [-171996, 0, -15576, -15576, -9182, 0, -7632, -7632], [0, -287428, -9864, 9864, 0, 10722, -7101, 7101], [-209752, 209752, 0, 14668, -4794, -4794, 0, -3331], [-442166, -442166, 45668, 0, 10380, -10380, -2098, 0], [-395216, -395216, 96644, 0, -16782, 16782, 8977, 0], [0, 83468, 15692, -15692, 0, 27598, -4955, 4955], [29220, 0, -12328, -12328, 3282, 0, 6845, 6845], [-613583, 0, 8680, 8680, 6997, 0, 2474, 2474], [540289, -540289, 0, -46716, 10614, 10614, 0, -2433], [357712, 357712, -51892, 0, 24156, -24156, -10364, 0], [0, 156636, 28568, -28568, 0, -6978, -280, 280], [0, 338002, -3296, 3296, 0, -21314, 3920, -3920], [-19878, 19878, 0, 53728, 3294, 3294, 0, -10109], [-425650, -425650, 18484, 0, -8490, 8490, 14225, 0], [0, -355736, -27284, 27284, 0, 6306, 8341, -8341], [-408283, 408283, 0, 11020, 4625, 4625, 0, -18724], [-200360, -200360, -63536, 0, -9792, 9792, -18305, 0], [-266532, 0, -133332, -133332, 514, 0, 1437, 1437], [237102, -237102, 0, 68260, 2550, 2550, 0, -17144], [166765, 166765, 38756, 0, -15363, 15363, -14480, 0], [570344, 570344, -36564, 0, -9132, 9132, -10368, 0], [108970, 0, -21596, -21596, 6038, 0, 8198, 8198], [0, -426778, 468, -468, 0, -15054, -558, 558], [0, 107460, -111000, 111000, 0, 24878, 7584, -7584], [390753, 390753, -6128, 0, 2214, -2214, 19234, 0], [-591419, 0, 41596, 41596, 19893, 0, -9241, -9241], [207218, 207218, 65764, 0, -6710, 6710, 10304, 0], [-322982, 322982, 0, -33772, -18108, -18108, 0, 11629], [-210073, -210073, -133212, 0, 8778, -8778, 21681, 0], [503417, 0, -79048, -79048, -36085, 0, 3661, 3661], [-223580, -223580, -211852, 0, -10188, 10188, 2185, 0], [-535889, 535889, 0, 96524, -1967, -1967, 0, 10258], [561028, 0, -34632, -34632, 23058, 0, -16245, -16245], [425130, 425130, 86848, 0, 24300, -24300, 1165, 0]]
